@@ -5,7 +5,8 @@
 #' 
 #' @name mlogit
 #' @aliases mlogit
-#' @import zoo
+#' @import Formula
+#' @import dfidx
 #' @importFrom stats as.formula coef dlnorm dnorm formula logLik
 #' @importFrom stats model.frame model.matrix model.response
 #' @importFrom stats na.omit pchisq plnorm pnorm predict
@@ -67,7 +68,7 @@
 #' @param ... further arguments passed to `mlogit.data` or
 #'     `mlogit.optim`.
 #' 
-#' @details For how to use the formula argument, see [mFormula()].
+#' @details For how to use the formula argument, see [Formula()].
 #' 
 #' The `data` argument may be an ordinary `data.frame`. In this case,
 #' some supplementary arguments should be provided and are passed to
@@ -114,7 +115,7 @@
 #' - freq: the frequency of choice,
 #' - residuals: the residuals,
 #' - fitted.values: the fitted values,
-#' - formula: the formula (a `mFormula` object),
+#' - formula: the formula (a `Formula` object),
 #' - expanded.formula: the formula (a `formula` object),
 #' - model: the model frame used,
 #' - index: the index of the choice and of the alternatives.
@@ -141,7 +142,7 @@
 #' ## charter
 #' 
 #' data("Fishing", package = "mlogit")
-#' Fish <- mlogit.data(Fishing, varying = c(2:9), shape = "wide", choice = "mode")
+#' Fish <- dfidx(Fishing, varying = 2:9, shape = "wide", choice = "mode")
 #' 
 #' ## a pure "conditional" model
 #' summary(mlogit(mode ~ price + catch, data = Fish))
@@ -150,18 +151,17 @@
 #' summary(mlogit(mode ~ 0 | income, data = Fish))
 #' 
 #' ## which can also be estimated using multinom (package nnet)
-#' library("nnet")
-#' summary(multinom(mode ~ income, data = Fishing))
+#' summary(nnet::multinom(mode ~ income, data = Fishing))
 #' 
 #' ## a "mixed" model
-#' m <- mlogit(mode ~ price+ catch | income, data = Fish)
+#' m <- mlogit(mode ~ price + catch | income, data = Fish)
 #' summary(m)
 #' 
 #' ## same model with charter as the reference level
-#' m <- mlogit(mode ~ price+ catch | income, data = Fish, reflevel = "charter")
+#' m <- mlogit(mode ~ price + catch | income, data = Fish, reflevel = "charter")
 #' 
 #' ## same model with a subset of alternatives : charter, pier, beach
-#' m <- mlogit(mode ~ price+ catch | income, data = Fish,
+#' m <- mlogit(mode ~ price + catch | income, data = Fish,
 #'             alt.subset = c("charter", "pier", "beach"))
 #' 
 #' ## model on unbalanced data i.e. for some observations, some
@@ -169,19 +169,16 @@
 #' # a data.frame in wide format with two missing prices
 #' Fishing2 <- Fishing
 #' Fishing2[1, "price.pier"] <- Fishing2[3, "price.beach"] <- NA
-#' mlogit(mode~price+catch|income, Fishing2, shape="wide", choice="mode", varying = 2:9)
+#' mlogit(mode ~ price + catch | income, Fishing2, shape = "wide", varying = 2:9)
 #' 
 #' # a data.frame in long format with three missing lines
 #' data("TravelMode", package = "AER")
 #' Tr2 <- TravelMode[-c(2, 7, 9),]
-#' mlogit(choice~wait+gcost|income+size, Tr2, shape = "long",
-#'        chid.var = "individual", alt.var="mode", choice = "choice")
+#' mlogit(choice ~ wait + gcost | income + size, Tr2)
 #' 
 #' ## An heteroscedastic logit model
 #' data("TravelMode", package = "AER")
-#' hl <- mlogit(choice ~ wait + travel + vcost, TravelMode,
-#'              shape = "long", chid.var = "individual", alt.var = "mode",
-#'              method = "bfgs", heterosc = TRUE, tol = 10)
+#' hl <- mlogit(choice ~ wait + travel + vcost, TravelMode, heterosc = TRUE)
 #' 
 #' ## A nested logit model
 #' TravelMode$avincome <- with(TravelMode, income * (mode == "air"))
@@ -190,25 +187,22 @@
 #' TravelMode$income <- with(TravelMode, income / 10)
 #' # Hensher and Greene (2002), table 1 p.8-9 model 5
 #' TravelMode$incomeother <- with(TravelMode, ifelse(mode %in% c('air', 'car'), income, 0))
-#' nl <- mlogit(choice~gcost+wait+incomeother, TravelMode,
-#'              shape='long', alt.var='mode',
-#'              nests=list(public=c('train', 'bus'), other=c('car','air')))
+#' nl <- mlogit(choice ~ gcost + wait + incomeother, TravelMode,
+#'              nests = list(public = c('train', 'bus'), other = c('car','air')))
+#'              
 #' # same with a comon nest elasticity (model 1)
 #' nl2 <- update(nl, un.nest.el = TRUE)
 #' 
 #' ## a probit model
 #' \dontrun{
-#' pr <- mlogit(choice ~ wait + travel + vcost, TravelMode,
-#'              shape = "long", chid.var = "individual", alt.var = "mode",
-#'              probit = TRUE)
+#' pr <- mlogit(choice ~ wait + travel + vcost, TravelMode, probit = TRUE)
 #' }
 #' 
 #' ## a mixed logit model
 #' \dontrun{
-#' rpl <- mlogit(mode ~ price+ catch | income, Fishing, varying = 2:9,
-#'               shape = 'wide', rpar = c(price= 'n', catch = 'n'),
-#'               correlation = TRUE, halton = NA,
-#'               R = 10, tol = 10, print.level = 0)
+#' rpl <- mlogit(mode ~ price + catch | income, Fishing, varying = 2:9,
+#'               rpar = c(price= 'n', catch = 'n'), correlation = TRUE,
+#'               alton = NA, R = 50)
 #' summary(rpl)
 #' rpar(rpl)
 #' cor.mlogit(rpl)
@@ -219,19 +213,20 @@
 #' 
 #' # a ranked ordered model
 #' data("Game", package = "mlogit")
-#' g <- mlogit(ch~own|hours, Game, choice='ch', varying = 1:12,
-#'             ranked=TRUE, shape="wide", reflevel="PC")
-mlogit <- function(formula, data, subset, weights, na.action, start= NULL,
-                   alt.subset = NULL, reflevel= NULL,
+#' g <- mlogit(ch ~ own | hours, Game, varying = 1:12, ranked = TRUE,
+#'             reflevel = "PC", idnames = c("chid", "alt"))
+mlogit <- function(formula, data, subset, weights, na.action, start = NULL,
+                   alt.subset = NULL, reflevel = NULL,
                    nests = NULL, un.nest.el = FALSE, unscaled = FALSE,
                    heterosc = FALSE, rpar = NULL, probit = FALSE,
                    R = 40, correlation = FALSE, halton = NULL, random.nb = NULL,
                    panel = FALSE, estimate = TRUE, seed = 10, ...){
     callT <- match.call(expand.dots = TRUE)
-    formula <- callT$formula <- mFormula(formula)
+    formula <- callT$formula <- Formula(formula)
     nframe <- length(sys.calls())
     start.time <- proc.time()
-    
+    mldata <- callT
+
     # 1 ######################################################
     # Check what kind of model is estimated
     ##########################################################
@@ -245,61 +240,141 @@ mlogit <- function(formula, data, subset, weights, na.action, start= NULL,
     mixed.logit <- mt["mixed"]
     if (multinom.logit) callT$method <- 'nr'
 
-    
-    # 2 ######################################################
-    # Run mlogit.data if necessary
+    # 1 ######################################################
+    # Subset the data.frame if necessary
     ##########################################################
-    mldata <- callT
-    # check if any of the mlogit.data argument is present
-    m <- match(c("data", "choice", "shape", "varying", "sep",
-                 "alt.var", "chid.var", "alt.levels", "group.var",
-                 "opposite", "drop.index", "id.var", "ranked"),
-               names(mldata), 0L)
-    # if any, except data is present run mlogit.data
-    if (sum(m[- 1])){
-        # The choice argument is irrelevant, use the response
-        # indicated in the formula
-        mldata <- mldata[c(1L, m)]
-        mldata$choice <- paste(deparse(attr(formula, "lhs")[[1]]))
-        mldata[[1L]] <- as.name("mlogit.data")
-        data <- eval(mldata, parent.frame())
+
+    # Subset is an argument of mlogit and dfidx. In any case, it is
+    # simpler to subset the df from mlogit, the df being either an
+    # ordinary df or a dfidx ; if the data argument is an ordinary
+    # data.frame, call dfidx without the subset argument
+
+
+    if ("subset" %in% names(mldata)){
+        sub_call <- mldata
+        m <- match(c("data", "subset"), names(sub_call), 0)
+        sub_call <- sub_call[c(1, m)]
+        names(sub_call)[2] <- "x"
+        sub_call[[1]] <- as.name("subset")
+        data <- eval(sub_call, parent.frame())
     }
+    # 2 ################################################################
+    # Run dfidx (or mlogit.data for backward compatibility) if necessary
+    ####################################################################
+    # check if any of the mlogit.data argument is present
     
+    common_args <- c("data", "drop.index", "shape",
+                     "choice", "varying", "sep", "opposite", "ranked")
+    dfidx_args <- c("idx", "as.factor", "pkg", "fancy.row.names", "idnames", "levels")
+    mlogit.data_args <- c("alt.var", "chid.var", "alt.levels", "id.var", "group.var")
+    match_dfidx <- match(dfidx_args, names(mldata), 0L)
+    match_mlogit.data <- match(mlogit.data_args, names(mldata), 0L)
+    match_common <- match(common_args, names(mldata), 0L)
+    to_dfidx <- any(as.logical(match_dfidx))
+    to_mlogit.data <- any(as.logical(match_mlogit.data))
+    # remove the first element which matches with data
+    to_common <- any(as.logical(match_common[-c(1:2)]))
+    # is_data.frame is TRUE if the data frame is not a dfidx object
+    is_data.frame <- ! inherits(data, "dfidx") & ! inherits(data, "mlogit.data")
+
+    # if the data frame is already a dfidx and some dfidx arguments
+    # are set, stop
+    if ((to_dfidx | to_mlogit.data | to_common) & ! is_data.frame)
+        stop("data is already a dfidx object")
+
+    # if dfidx and mlogit arguments are mixed, the stop
+    if (to_dfidx & to_mlogit.data)
+        stop("some specificic arguments of mlogit.data and dfidx are introduced")
+
+    # it data is an ordinary data.frame, run dfidx or mlogit.data
+
+    if (is_data.frame){
+        if (to_mlogit.data){
+#            warning("the use of mlogit.data is deprecated, use dfidx' arguments instead")
+            mldata[[1L]] <- as.name("mlogit.data")
+            m <- match(c(common_args, mlogit.data_args),
+                       names(mldata), 0L)
+        }
+        else{
+            mldata$pkg <- "mlogit"
+            mldata[[1L]] <- as.name("dfidx")
+            m <- match(c(common_args, dfidx_args),
+                       names(mldata), 0L)
+            to_dfidx <- TRUE
+        }
+        if (to_mlogit.data | to_dfidx){
+            # The choice argument is irrelevant, use the response
+            # indicated in the formula
+            mldata <- mldata[c(1L, m)]
+            mldata$choice <- paste(deparse(attr(formula, "lhs")[[1]]))
+            # This works only if dfidx is attached, which is not the
+            # case in some packages which imports (but not depends on)
+            # mlogit 
+            if (to_mlogit.data) data <- eval(mldata, parent.frame())
+            else{
+            # Construct a list of dfidx' arguments with the default values            
+                dfa <- list(data = NA, idx = NULL, drop.index = TRUE, as.factor = NULL, pkg = NULL,
+                            fancy.row.names = FALSE,
+                            idnames = NULL, shape = "long", choice = NULL,
+                            varying = NULL, sep = ".", opposite = NULL, levels = NULL, ranked = FALSE)
+                # Replace the relevant values by those indicated in mlogit
+                dfa[names(mldata)[-1]] <- as.list(mldata)[- 1]
+                # Run dfidx with these values
+                data <- dfidx::dfidx(data = data, dfa$idx, drop.index = dfa$drop.index,
+                                     as.factor = dfa$as.factor,
+                                     pkg = dfa$pkg, fancy.row.names = dfa$fancy.row.names,
+                                     idnames = dfa$idnames, shape = dfa$shape,
+                                     choice = dfa$choice, varying = dfa$varying,
+                                     sep = dfa$sep, opposite = dfa$opposite,
+                                     levels = dfa$levels, ranked = dfa$ranked)
+            }
+        }
+    }
+    # if data is an ordinary dfidx object, add the dfidx_mlogit class
+    if (class(data)[1] == "dfidx") class(data) <- c("dfidx_mlogit", class(data))
+
     # 3 ######################################################
     # compute the model.frame
     ##########################################################
-    
+
     mf <- callT
     mf$data <- data
     m <- match(c("formula", "data", "subset", "na.action",
                  "weights", "alt.subset", "reflevel"),
                names(mf), 0L)
     mf <- mf[c(1L, m)]
-    mf$formula <- formula
-    mf$data <- data
+    # use the unusual order of the arguments of model.frame, first
+    # data, then formula
+    mf$formula <- data
+    mf$data <- formula
     mf[[1L]] <- as.name("model.frame")
+    # mlogit needs balanced data
+    mf$balanced <- TRUE
     mf <- eval(mf, parent.frame())
 
     # 4 ###########################################################
     # get the dimensions of the model
     ###############################################################
-    index <- index(mf)
-    alt <- index$alt
-    chid <- index$chid
-    alt.lev <- levels(alt)
-    if (is.null(attr(index, "alt.ordering"))) alt.ord <- levels(alt)
-    else alt.ord <- attr(index, "alt.ordering")
-    altnoNA <- rep(alt.ord, nrow(index) / length(alt.lev))
+    .idx <- dfidx::idx(mf)
+    alt <- dfidx::idx(mf, 2)
+    chid <- dfidx::idx(mf, 1)
+    alt.lev <- levels(alt[drop = TRUE])
+    # model.frame_dfidx returns an alt.ordering attribute which
+    # contains the levels of the second index in their initial order,
+    # i.e. before redefining the reference level
+    if (is.null(attr(mf, "alt.ordering"))) alt.ord <- levels(alt)
+    else alt.ord <- attr(mf, "alt.ordering")
+    altnoNA <- rep(alt.ord, nrow(.idx) / length(alt.lev))
     altnoNA <- factor(altnoNA, levels = alt.lev, labels = alt.lev)
     J <- length(alt.lev)
-    n <- length(unique(chid))
-
+    N <- length(unique(chid))
     # 5 ###########################################################
     # extract the elements of the model
     ###############################################################
 
     # extract the weights if necessary
     if (any(names(mf) == "(weights)")){
+        #YC à voir bizarre
         weights <- matrix(mf[["(weights)"]], ncol = J, byrow = TRUE)
         weights <- as.numeric(apply(weights, 1, min, na.rm = TRUE))
         weights <- weights / mean(weights)
@@ -309,56 +384,67 @@ mlogit <- function(formula, data, subset, weights, na.action, start= NULL,
     # extract the individual index if it is relevant
     if (panel){
         if (! mixed.logit) stop("panel is only relevant for mixed logit models")
-        id <- index$id
+        id <- dfidx::idx(mf, 1, 2)
         if (is.null(id)) stop("no individual index")
-        id <- unique(index[, c("chid", "id")])$id
+        # construct a data.frame with all the unique chids and the corresponding id
+        #YC ????
+        id <- unique(data.frame(chid, id))$id
     }
     else id <- NULL
 
     # extract the X matrix for the standard deviation estimation
     # (cformula is used to save the potentially 4 part formula)
+
     cformula <- formula
     if (length(formula)[2] == 4){
+        #YC à revoir, incompréhensible
         # sformula is used to extract XS, formula is reduced to its
         # first 3 parts
-        sformula <- formula(as.Formula(formula), rhs = 4)
-        Xs <- model.matrix(sformula, mf);
+        sformula <- formula(formula, rhs = 4)
+        # ca semble marcher, mais pas top
+        Xs <- model.matrix(sformula, as.data.frame(mf))
+#        Xs <- model.matrix(sformula, mf)
         # only one line per choice situation is kept, one has to take
         # care to NA values due to the balancing of the data performed
         # by model.frame
         Xs <- na.omit(Xs)
         to.omit <- attr(Xs, "na.action")
         Xs <- Xs[! duplicated(chid[- to.omit]), - 1, drop = FALSE]
-        formula <- mFormula(formula(as.Formula(formula), rhs = 1:3))
+        # incompréhensible !!!!!!
+        formula <- Formula(formula(formula, rhs = 1:3))
+#        formula <- mFormula(formula(as.Formula(formula), rhs = 1:3))
     }
     else Xs <- NULL
-    X <- model.matrix(formula, mf)
+    attr(mf, "formula") <- formula
+    X <- model.matrix(mf, rhs = 1:3)
     formula <- cformula
     K <- ncol(X)
-    df.residual <- n - K
+    df.residual <- N - K
     colnamesX <- colnames(X)
 
-    # extract the response
-    y <- model.response(mf)
+    # extract the response and coerce it to a logical
+    y <- as.logical(model.response(mf))
+    # choice is a vector of length N containing the alternative chosen
     choice <- na.omit(alt[y])
-    
     # compute the choice frequency table
-    freq <- table(alt[y])
-    
-    # Xl and yl are lists of length J which contains n matrix / vector
-    # of covariates and response (a boolean) ; yv is a vector that
-    # contains the chosen alternative
-
+    freq <- table(alt[as.logical(y)])
+    #YC should do the same, much simplier
+    freq <- table(choice)
+    # Xl and yl are lists of length J which contains N matrix / vector
+    # of covariates and response; yv is a vector that contains the
+    # chosen alternative
     Xl <- vector(length = J, mode = "list")
     names(Xl) <- levels(alt)
     for (i in levels(alt))  Xl[[i]] <- X[altnoNA == i, , drop = FALSE]
     yl <- split(y, altnoNA)
     yl <- lapply(yl, function(x){x[is.na(x)] <- FALSE ; x})
-    attr(yl, "chid") <- as.character(levels(chid))
+    unchid <- if(is.factor(chid)) as.character(levels(chid)) else unique(chid)
+    attr(yl, "chid") <- unchid
     attr(yl, "id") <- as.character(levels(id))
     # for probit the response is a vector that contains the chosen
     # alternative as a numeric ; NA values of y are replaced by FALSE
     # so that the chosen alternative is correctly returned    
+
     if (probit){
         y2 <- y
         y2[is.na(y2)] <- FALSE
@@ -366,8 +452,8 @@ mlogit <- function(formula, data, subset, weights, na.action, start= NULL,
         # DX is a list of covariates first differenced with respect with the
         # chosen alternative
         DX <- vector("list", length = J - 1)
-        DX <- lapply(DX, function(x) return(matrix(NA, nrow = n, ncol = K)))
-        for (i in 1:n){
+        DX <- lapply(DX, function(x) return(matrix(NA, nrow = N, ncol = K)))
+        for (i in 1:N){
             any <- yv[i]
             j <- 1
             newj <- 1
@@ -382,14 +468,14 @@ mlogit <- function(formula, data, subset, weights, na.action, start= NULL,
 
     # if the nests are defined by a grouping variable or by 'rpl',
     # compute the corresponding list of nests
+
     if (nested.logit){
         if (is.logical(nests) && nests){
-            if (is.null(index$group)){
-                stop("no grouping variable")
-            }
+            group_var <- dfidx::idx(mf, 2, 2)
+            if (is.null(group_var)) stop("no grouping variable")
             else{
                 nests <- unique(data.frame(alt = as.character(alt),
-                                           group = index$group))
+                                           group = group_var))
                 nests <- split(nests$alt, nests$group)
             }
         }
@@ -401,7 +487,7 @@ mlogit <- function(formula, data, subset, weights, na.action, start= NULL,
             names(nests) <- names.nests
         }
     }
-    
+
     # 6 ######################################################
     # compute the starting values
     ##########################################################
@@ -446,7 +532,7 @@ mlogit <- function(formula, data, subset, weights, na.action, start= NULL,
     opt <- opt[c(1L, m)]
     opt[[1]] <- as.name('mlogit.optim')
     opt$logLik <- as.name('lnl.slogit')
-    opposite <- - 1 
+    opposite <- - 1
     opt[c('weights', 'opposite')] <- list(as.name('weights'), as.name('opposite'))
 
     # model specific arguments
@@ -477,9 +563,9 @@ mlogit <- function(formula, data, subset, weights, na.action, start= NULL,
         opt[c('logLik', 'nests', 'un.nest.el', 'unscaled')] <-
             list(as.name('lnl.nlogit'), as.name('nests'),
                  as.name('un.nest.el'), as.name('unscaled'))
-    
-    x <- eval(opt, sys.frame(which = nframe))
 
+    x <- eval(opt, sys.frame(which = nframe))
+    
     # compute the probabilities for all the alternatives for
     # heteroscedastic and the probit model
     if (probit | heterosc){
@@ -492,11 +578,11 @@ mlogit <- function(formula, data, subset, weights, na.action, start= NULL,
         probabilities <- c()
         for (k in 1:J){
             if (probit){
-                they <- rep(k, n)
+                they <- rep(k, N)
                 opt$y <- they
                 DX <- vector("list", length = J - 1)
-                DX <- lapply(DX, function(x) return(matrix(NA, nrow = n, ncol = K)))
-                for (i in 1:n){
+                DX <- lapply(DX, function(x) return(matrix(NA, nrow = N, ncol = K)))
+                for (i in 1:N){
                     any <- k
                     j <- 1
                     newj <- 1
@@ -510,9 +596,9 @@ mlogit <- function(formula, data, subset, weights, na.action, start= NULL,
                 opt$X <- DX
             }
             if (heterosc){
-                they <- vector(mode='list', length= J)
-                they <- lapply(they, function(x) rep(FALSE, n))
-                they[[k]] <- rep(TRUE, n)
+                they <- vector(mode = 'list', length = J)
+                they <- lapply(they, function(x) rep(FALSE, N))
+                they[[k]] <- rep(TRUE, N)
                 names(they) <- names(yl)
                 opt$y <- they
             }
@@ -522,20 +608,20 @@ mlogit <- function(formula, data, subset, weights, na.action, start= NULL,
         colnames(probabilities) <- alt.lev
         attr(x$optimum, "probabilities") <- probabilities
     }
-    
+
     # 8 ###########################################################
     # put the result in form
     ###############################################################
 
     # some general features
+    #YC n/N already defined
     n <- sum(freq)
     x$est.stat$elaps.time <- proc.time() - start.time
     logLik <- structure( - as.numeric(x$optimum),
                         df = length(x$coefficients),
-                        null = sum(freq * log(freq / n)),
+                        null = sum(freq * log(freq / N)),
                         class = "logLik"
                         )
-
     if (mixed.logit) rpar <- make.rpar(rpar, correlation, x$coefficients, NULL) else rpar <- NULL
 #    if (! nested.logit) nests <- NULL
 
@@ -560,7 +646,7 @@ mlogit <- function(formula, data, subset, weights, na.action, start= NULL,
     }
     if (probit){
         S <- matrix(0, J - 1, J - 1)
-        S[!upper.tri(S)] <- x$coefficients[-c(1:K)]
+        S[! upper.tri(S)] <- x$coefficients[- c(1:K)]
         Mi <- list()
         M <- rbind(0, diag(J - 2))
         Mi[[1]] <- diag(J - 1)
@@ -583,33 +669,28 @@ mlogit <- function(formula, data, subset, weights, na.action, start= NULL,
             alts <- nests[[i]]
             M <- length(alts)
             for (m in 1:M){
-                for (n in 1:M){
-                    Omega[alts[m], alts[n]] <- (1 - anEl^2)
+                for (l in 1:M){
+                    Omega[alts[m], alts[l]] <- (1 - anEl ^ 2)
                 }
             }
         }
         diag(Omega) <- 1
-        Omega <- Omega * tcrossprod(rep(pi/sqrt(6), J))
+        Omega <- Omega * tcrossprod(rep(pi / sqrt(6), J))
     }
     if (heterosc.logit){
         alt.names <- colnames(probabilities)
         J <- length(alt.names)
         Omega <- matrix(0, J, J, dimnames = list(alt.names, alt.names))
-        diag(Omega) <- pi^2 / 6
+        diag(Omega) <- pi ^ 2 / 6
         for (i in 2:J){
             analt <- alt.names[i]
-            Omega[analt, analt] <- pi ^ 2 / 6 * x$coefficients[paste('sp', analt, sep = '.')]^2
+            Omega[analt, analt] <- pi ^ 2 / 6 * x$coefficients[paste('sp', analt, sep = '.')] ^ 2
         }
     }
     if (wlogit) Omega <- NA
 
-    mfindex <- index(mf)
     mf$probabilities <- as.numeric(t(probabilities))
     if (! is.null(linpred)) mf$linpred <- as.numeric(t(linpred))
-    mf <- structure(mf,
-                    class = c("mlogit.data", "data.frame"),
-                    index = mfindex)
-
     result <- structure(
         list(
             coefficients  = x$coefficients,
@@ -679,31 +760,35 @@ mlogit.start <- function(formula, data, mf, start = NULL, un.nest.el = FALSE,
     heterosc.logit <- mt['heterosc']
     mixed.logit <- mt['mixed']
 
+    alt <- dfidx::idx(mf, 2)
+    
     # extract the X and eventually Xs matrix
     if (length(formula)[2] == 4){
         sformula <- formula(as.Formula(formula), rhs = 4)
-        Xs <- model.matrix(sformula, mf)[! duplicated(index(mf)$chid), - 1]
-        formula <- mFormula(formula(as.Formula(formula), rhs = 1:3))
+        Xs <- model.matrix(sformula, as.data.frame(mf))[! duplicated(dfidx::idx(mf, 1)), - 1]
+#        Xs <- model.matrix(sformula, mf)[! duplicated(index(mf)$chid), - 1]
+#        formula <- mFormula(formula(as.Formula(formula), rhs = 1:3))
+        formula <- Formula(formula(formula, rhs = 1:3))
     }
     else Xs <- NULL
-    X <- model.matrix(formula, mf)
-    alt.lev <- levels(index(mf)$alt)
+    alt.lev <- levels(alt)
     J <- length(alt.lev)
+    attr(mf, "formula") <- formula
+    X <- model.matrix(mf)
     K <- ncol(X)
     colnamesX <- colnames(X)
-    
     # give names to the supplementary coefficients and values if start
     # is null or of length K
     sup.coef <- numeric(0)
     names.sup.coef <- character(0)
     if (nested.logit){
         if (is.logical(nests) && nests){
-            if (is.null(index(mf)$group)){
+            if (is.null(dfidx::idx(mf, 2, 2))){
                 stop("no grouping variable")
             }
             else{
-                nests <- unique(data.frame(alt = as.character(index(mf)$alt),
-                                           group = index(mf)$group))
+                nests <- unique(data.frame(alt = as.character(dfidx::idx(mf, 2)),
+                                           group = dfidx::idx(mf, 2, 2)))
                 nests <- split(nests$alt, nests$group)
             }
         }
@@ -716,7 +801,8 @@ mlogit.start <- function(formula, data, mf, start = NULL, un.nest.el = FALSE,
         }
         else{
             if (is.null(start) || length(start) == K) sup.coef <- rep(1, L)
-            names.sup.coef <- paste("iv", names(nests), sep = ":")
+#            names.sup.coef <- paste(names(nests), 'iv', sep = ":")
+            names.sup.coef <- paste('iv', names(nests), sep = ":")            
         }
     }
     if (pair.comb.logit){
@@ -824,8 +910,9 @@ mlogit.start <- function(formula, data, mf, start = NULL, un.nest.el = FALSE,
             callst[c("correlation", "un.nest.el")] <- FALSE
             callst$print.level <- 0
             if (length(callst$formula)[2] == 4)
-                callst$formula <- mFormula(formula(callst$formula, rhs = 1:3))
-            callst[[1]] <- as.name("mlogit")
+#                callst$formula <- mFormula(formula(callst$formula, rhs = 1:3))
+                callst$formula <- Formula(formula(callst$formula, rhs = 1:3))
+                callst[[1]] <- as.name("mlogit")
             start <- coef(eval(callst, parent.frame()))
             if (mixed.logit){
                 ln <- names(rpar[rpar == "ln"])
