@@ -6,13 +6,11 @@ options(width = 65)
 library("mlogit")
 data("ModeCanada", package = "mlogit")
 busUsers <- with(ModeCanada, case[choice == 1 & alt == 'bus'])
-Bhat <- subset(ModeCanada, !case %in% busUsers & alt != 'bus' & noalt == 4)
+Bhat <- subset(ModeCanada, ! case %in% busUsers & alt != 'bus' & noalt == 4)
 Bhat$alt <- Bhat$alt[drop = TRUE]
-Bhat <- mlogit.data(Bhat, shape='long', chid.var = 'case',
-                    alt.var = 'alt', choice='choice',
-                    drop.index=TRUE)
-pcl <- mlogit(choice~freq+cost+ivt+ovt, Bhat, reflevel='car',
-              nests='pcl', constPar=c('iv:train.air'))
+Bhat <- dfidx(Bhat, idx = c("case", "alt"), choice = "choice", idnames = c("chid", "alt"))
+pcl <- mlogit(choice ~ freq + cost + ivt + ovt, Bhat, reflevel = 'car',
+              nests = 'pcl', constPar=c('iv:train.air'))
 summary(pcl)
 
 ## --------------------------------------------------------------
@@ -24,8 +22,9 @@ nrow(Game)
 nrow(Game2)
 
 ## --------------------------------------------------------------
-G <- mlogit.data(Game2, shape = "long", choice = "ch", alt.var = 'platform', ranked = TRUE)
-G <- mlogit.data(Game, shape = "wide", choice = "ch", varying = 1:12, ranked = TRUE)
+G <- dfidx(Game, varying = 1:12, choice = "ch", ranked = TRUE, idnames = c("chid", "alt"))
+G <- dfidx(Game2, choice = "ch", ranked = TRUE, idx = c("chid", "platform"),
+           idnames = c("chid", "alt"))
 head(G)
 nrow(G)
 
